@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -421,6 +421,7 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  exception=(&image->exception);
   data_size=(unsigned char) ReadBlobByte(image);
   if (data_size > MaximumLZWBits)
     ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
@@ -428,7 +429,6 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
   if (lzw_info == (LZWInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  exception=(&image->exception);
   pass=0;
   offset=0;
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -948,17 +948,22 @@ static MagickBooleanType PingGIFImage(Image *image)
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (ReadBlob(image,1,&data_size) != 1)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   if (data_size > MaximumLZWBits)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   if (ReadBlob(image,1,&length) != 1)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   while (length != 0)
   {
     if (ReadBlob(image,length,buffer) != (ssize_t) length)
-      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+        image->filename);
     if (ReadBlob(image,1,&length) != 1)
-      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+        image->filename);
   }
   return(MagickTrue);
 }
@@ -1041,6 +1046,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   meta_image=AcquireImage(image_info);  /* metadata container */
   meta_image->page.width=ReadBlobLSBShort(image);
   meta_image->page.height=ReadBlobLSBShort(image);
+  meta_image->iterations=1;
   flag=(unsigned char) ReadBlobByte(image);
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
@@ -1098,19 +1104,28 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               *comments;
 
             size_t
-              length;
+              extent,
+              offset;
 
-            /*
-              Read comment extension.
-            */
             comments=AcquireString((char *) NULL);
-            for (length=0; ; length+=count)
+            extent=MagickPathExtent;
+            for (offset=0; ; offset+=count)
             {
               count=ReadBlobBlock(image,buffer);
               if (count == 0)
                 break;
               buffer[count]='\0';
-              (void) ConcatenateString(&comments,(const char *) buffer);
+              if (((ssize_t) (count+offset+MagickPathExtent)) >= (ssize_t) extent)
+                {
+                  extent<<=1;
+                  comments=(char *) ResizeQuantumMemory(comments,extent+
+                    MagickPathExtent,sizeof(*comments));
+                  if (comments == (char *) NULL)
+                    ThrowGIFException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                }
+              (void) CopyMagickString(&comments[offset],(char *) buffer,extent-
+                offset);
             }
             (void) SetImageProperty(meta_image,"comment",comments);
             comments=DestroyString(comments);
@@ -1275,7 +1290,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ((size_t) (flag & 0x07)+1);
     image->colors=local_colors;
     if (opacity >= (ssize_t) image->colors)
-      image->colors=(size_t) (opacity+1);
+      {
+        image->colors++;
+        opacity=(-1);
+      }
     image->ticks_per_second=100;
     image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
     if ((image->columns == 0) || (image->rows == 0))
@@ -1731,8 +1749,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
         value=GetImageProperty(image,"comment");
-        if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-            (value != (const char *) NULL))
+        if (value != (const char *) NULL)
           {
             register const char
               *p;
