@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -328,7 +328,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     if ((bits_per_pixel != 1) && (bits_per_pixel != 2) &&
         (bits_per_pixel != 4) && (bits_per_pixel != 8) &&
         (bits_per_pixel != 16))
-      ThrowReaderException(CorruptImageError,"UnrecognizedBitsPerPixel");
+      ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
     version=(size_t) ReadBlobByte(image);
     if ((version != 0) && (version != 1) && (version != 2))
       ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch");
@@ -374,7 +374,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
             for (i=0; i < (ssize_t) count; i++)
             {
               ReadBlobByte(image);
-              index=ConstrainColormapIndex(image,(size_t) (255-i));
+              index=ConstrainColormapIndex(image,(ssize_t) (255-i));
               image->colormap[(int) index].red=ScaleCharToQuantum(
                 (unsigned char) ReadBlobByte(image));
               image->colormap[(int) index].green=ScaleCharToQuantum(
@@ -386,7 +386,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       else
         for (i=0; i < (ssize_t) (1L << bits_per_pixel); i++)
         {
-          index=ConstrainColormapIndex(image,(size_t) (255-i));
+          index=ConstrainColormapIndex(image,(ssize_t) (255-i));
           image->colormap[(int) index].red=ScaleCharToQuantum(
             PalmPalette[i][0]);
           image->colormap[(int) index].green=ScaleCharToQuantum(
@@ -404,7 +404,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         image->storage_class=PseudoClass;
         image->depth=8;
       }
-    if (image_info->ping != MagickFalse) 
+    if (image_info->ping != MagickFalse)
       {
         (void) CloseBlob(image);
         return(image);
@@ -419,7 +419,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     one_row=(unsigned char *) AcquireQuantumMemory(extent,sizeof(*one_row));
     if (one_row == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    (void) ResetMagickMemory(one_row,0,extent*sizeof(*one_row));
+    (void) memset(one_row,0,extent*sizeof(*one_row));
     last_row=(unsigned char *) NULL;
     if (compressionType == PALM_COMPRESSION_SCANLINE)
       {
@@ -430,6 +430,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
             one_row=(unsigned char *) RelinquishMagickMemory(one_row);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
+         (void) memset(last_row,0,MagickMax(bytes_per_row,2*image->columns)*
+           sizeof(*last_row));
       }
     mask=(size_t) (1U << bits_per_pixel)-1;
     for (y=0; y < (ssize_t) image->rows; y++)
@@ -455,7 +457,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
                   break;
                 count=MagickMin(count,(ssize_t) bytes_per_row-i);
                 byte=(size_t) ReadBlobByte(image);
-                (void) ResetMagickMemory(one_row+i,(int) byte,(size_t) count);
+                (void) memset(one_row+i,(int) byte,(size_t) count);
                 i+=count;
               }
           }
@@ -482,7 +484,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
                     one_row[i+bit]=last_row[i+bit];
                 }
               }
-              (void) CopyMagickMemory(last_row, one_row, bytes_per_row);
+              (void) memcpy(last_row, one_row, bytes_per_row);
             }
         }
       ptr=one_row;
@@ -587,8 +589,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            (void) DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -598,6 +600,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       }
   } while (nextDepthOffset != 0);
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -726,6 +730,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     count,
     bits_per_pixel,
     bytes_per_row,
+    imageListLength,
     nextDepthOffset,
     one;
 
@@ -769,6 +774,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
   one=1;
   version=0;
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     (void) TransformImageColorspace(image,sRGBColorspace);
@@ -893,7 +899,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     for (y=0; y < (ssize_t) image->rows; y++)
     {
       ptr=one_row;
-      (void) ResetMagickMemory(ptr,0,bytes_per_row);
+      (void) memset(ptr,0,bytes_per_row);
       p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
       if (p == (PixelPacket *) NULL)
         break;
@@ -978,7 +984,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
               (void) WriteBlobByte(image, byte);
               (void) WriteBlob(image,tptr-tmpbuf,(unsigned char *) tmpbuf);
             }
-            (void) CopyMagickMemory(last_row,one_row,bytes_per_row);
+            (void) memcpy(last_row,one_row,bytes_per_row);
           }
         else
           (void) WriteBlob(image,bytes_per_row,one_row);
@@ -1023,8 +1029,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
     currentOffset=(MagickOffsetType) GetBlobSize(image);
     (void) SeekBlob(image,currentOffset,SEEK_SET);
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
