@@ -17,13 +17,13 @@
 %                                September 2002                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -57,6 +57,7 @@
 #include "magick/token.h"
 #include "magick/thread_.h"
 #include "magick/thread-private.h"
+#include "magick/timer-private.h"
 #include "magick/utility.h"
 #include "magick/utility-private.h"
 #include "magick/version.h"
@@ -96,7 +97,7 @@ typedef struct _EventInfo
 typedef struct _HandlerInfo
 {
   const char
-    *name;
+    name[10];
 
   LogHandlerType
     handler;
@@ -133,11 +134,14 @@ struct _LogInfo
   TimerInfo
     timer;
 
-  size_t
-    signature;
-
   MagickLogMethod
     method;
+
+  SemaphoreInfo
+    *event_semaphore;
+
+  size_t
+    signature;
 };
 
 typedef struct _LogMapInfo
@@ -166,31 +170,31 @@ static const HandlerInfo
     { "None", NoHandler },
     { "Stderr", StderrHandler },
     { "Stdout", StdoutHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler },
-    { (char *) NULL, UndefinedHandler }
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler },
+    { "", UndefinedHandler }
   };
 
 static const LogMapInfo
@@ -210,20 +214,19 @@ static MagickBooleanType
   event_logging = MagickFalse;
 
 static SemaphoreInfo
-  *event_semaphore = (SemaphoreInfo *) NULL,
   *log_semaphore = (SemaphoreInfo *) NULL;
 
 /*
   Forward declarations.
 */
 static LogHandlerType
-  ParseLogHandlers(const char *);
+  ParseLogHandlers(const char *) magick_attribute((__pure__));
 
 static LogInfo
   *GetLogInfo(const char *,ExceptionInfo *);
 
 static MagickBooleanType
-  IsLogCacheInstantiated(ExceptionInfo *),
+  IsLogCacheInstantiated(ExceptionInfo *) magick_attribute((__pure__)),
   LoadLogCache(LinkedListInfo *,const char *,const char *,const size_t,
     ExceptionInfo *);
 
@@ -672,7 +675,8 @@ static inline void CheckEventLogging()
 
       ResetLinkedListIterator(log_cache);
       p=(LogInfo *) GetNextValueInLinkedList(log_cache);
-      event_logging=p->event_mask != NoEvents ? MagickTrue: MagickFalse;
+      event_logging=(p != (LogInfo *) NULL) && (p->event_mask != NoEvents) ? 
+        MagickTrue: MagickFalse;
     }
 }
 
@@ -784,7 +788,7 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
           size_t
             mask;
 
-          if (LogHandlers[j].name == (const char *) NULL)
+          if (*LogHandlers[j].name == '\0')
             break;
           mask=1;
           mask<<=j;
@@ -846,7 +850,6 @@ MagickExport MagickBooleanType LogComponentGenesis(void)
   exception=AcquireExceptionInfo();
   (void) GetLogInfo("*",exception);
   exception=DestroyExceptionInfo(exception);
-  event_semaphore=AllocateSemaphoreInfo();
   return(MagickTrue);
 }
 
@@ -887,17 +890,14 @@ static void *DestroyLogElement(void *log_info)
     p->path=DestroyString(p->path);
   if (p->filename != (char *) NULL)
     p->filename=DestroyString(p->filename);
+  if (p->event_semaphore != (SemaphoreInfo *) NULL)
+    DestroySemaphoreInfo(&p->event_semaphore);
   p=(LogInfo *) RelinquishMagickMemory(p);
   return((void *) NULL);
 }
 
 MagickExport void LogComponentTerminus(void)
 {
-  if (event_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&event_semaphore);
-  LockSemaphoreInfo(event_semaphore);
-  UnlockSemaphoreInfo(event_semaphore);
-  DestroySemaphoreInfo(&event_semaphore);
   if (log_semaphore == (SemaphoreInfo *) NULL)
     ActivateSemaphoreInfo(&log_semaphore);
   LockSemaphoreInfo(log_semaphore);
@@ -975,7 +975,7 @@ static char *TranslateEvent(const LogEventType magick_unused(type),
   exception=AcquireExceptionInfo();
   log_info=(LogInfo *) GetLogInfo("*",exception);
   exception=DestroyExceptionInfo(exception);
-  seconds=time((time_t *) NULL);
+  seconds=GetMagickTime();
   elapsed_time=GetElapsedTime(&log_info->timer);
   user_time=GetUserTime(&log_info->timer);
   text=AcquireString(event);
@@ -1290,12 +1290,12 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
   exception=AcquireExceptionInfo();
   log_info=(LogInfo *) GetLogInfo("*",exception);
   exception=DestroyExceptionInfo(exception);
-  if (event_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&event_semaphore);
-  LockSemaphoreInfo(event_semaphore);
+  if (log_info->event_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&log_info->event_semaphore);
+  LockSemaphoreInfo(log_info->event_semaphore);
   if ((log_info->event_mask & type) == 0)
     {
-      UnlockSemaphoreInfo(event_semaphore);
+      UnlockSemaphoreInfo(log_info->event_semaphore);
       return(MagickTrue);
     }
   domain=CommandOptionToMnemonic(MagickLogEventOptions,type);
@@ -1310,7 +1310,7 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
   if (text == (char *) NULL)
     {
       (void) ContinueTimer((TimerInfo *) &log_info->timer);
-      UnlockSemaphoreInfo(event_semaphore);
+      UnlockSemaphoreInfo(log_info->event_semaphore);
       return(MagickFalse);
     }
   if ((log_info->handler_mask & ConsoleHandler) != 0)
@@ -1339,7 +1339,7 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
       file_info.st_size=0;
       if (log_info->file != (FILE *) NULL)
         (void) fstat(fileno(log_info->file),&file_info);
-      if (file_info.st_size > (ssize_t) (1024*1024*log_info->limit))
+      if (file_info.st_size > (MagickOffsetType) (1024*1024*log_info->limit))
         {
           (void) FormatLocaleFile(log_info->file,"</log>\n");
           (void) fclose(log_info->file);
@@ -1354,7 +1354,7 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
           if (filename == (char *) NULL)
             {
               (void) ContinueTimer((TimerInfo *) &log_info->timer);
-              UnlockSemaphoreInfo(event_semaphore);
+              UnlockSemaphoreInfo(log_info->event_semaphore);
               return(MagickFalse);
             }
           log_info->append=IsPathAccessible(filename);
@@ -1362,7 +1362,7 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
           filename=(char  *) RelinquishMagickMemory(filename);
           if (log_info->file == (FILE *) NULL)
             {
-              UnlockSemaphoreInfo(event_semaphore);
+              UnlockSemaphoreInfo(log_info->event_semaphore);
               return(MagickFalse);
             }
           log_info->generation++;
@@ -1391,7 +1391,7 @@ MagickExport MagickBooleanType LogMagickEventList(const LogEventType type,
     }
   text=(char  *) RelinquishMagickMemory(text);
   (void) ContinueTimer((TimerInfo *) &log_info->timer);
-  UnlockSemaphoreInfo(event_semaphore);
+  UnlockSemaphoreInfo(log_info->event_semaphore);
   return(MagickTrue);
 }
 
@@ -1698,7 +1698,7 @@ static LogHandlerType ParseLogHandlers(const char *handlers)
     while ((*p != '\0') && ((isspace((int) ((unsigned char) *p)) != 0) ||
            (*p == ',')))
       p++;
-    for (i=0; LogHandlers[i].name != (char *) NULL; i++)
+    for (i=0; *LogHandlers[i].name != '\0'; i++)
     {
       length=strlen(LogHandlers[i].name);
       if (LocaleNCompare(p,LogHandlers[i].name,length) == 0)
@@ -1707,7 +1707,7 @@ static LogHandlerType ParseLogHandlers(const char *handlers)
           break;
         }
     }
-    if (LogHandlers[i].name == (char *) NULL)
+    if (*LogHandlers[i].name == '\0')
       return(UndefinedHandler);
   }
   return(handler_mask);
