@@ -17,13 +17,13 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -246,6 +246,21 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   const MagickRealType weighting_exponent,const MagickBooleanType verbose)
 {
 #define SegmentImageTag  "Segment/Image"
+#define ThrowClassifyException(severity,tag,label) \
+{\
+  for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster) \
+  { \
+    next_cluster=cluster->next; \
+    cluster=(Cluster *) RelinquishMagickMemory(cluster); \
+  } \
+  if (squares != (double *) NULL) \
+    { \
+      squares-=255; \
+      free_squares=squares; \
+      free_squares=(double *) RelinquishMagickMemory(free_squares); \
+    } \
+  ThrowBinaryException(severity,tag,label); \
+}
 
   CacheView
     *image_view;
@@ -291,9 +306,11 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  squares=(double *) NULL;
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
+  exception=(&image->exception);
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -317,7 +334,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
             head=cluster;
           }
         if (cluster == (Cluster *) NULL)
-          ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
             image->filename);
         /*
           Initialize a new class.
@@ -337,7 +354,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       */
       cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
       if (cluster == (Cluster *) NULL)
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
       /*
         Initialize a new class.
@@ -355,7 +372,6 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   status=MagickTrue;
   count=0;
   progress=0;
-  exception=(&image->exception);
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -403,10 +419,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,
-          2*image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -499,13 +515,13 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       (void) FormatLocaleFile(stdout,"\n");
     }
   if (number_clusters > 256)
-    ThrowBinaryException(ImageError,"TooManyClusters",image->filename);
+    ThrowClassifyException(ImageError,"TooManyClusters",image->filename);
   /*
     Speed up distance calculations.
   */
   squares=(MagickRealType *) AcquireQuantumMemory(513UL,sizeof(*squares));
   if (squares == (MagickRealType *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   squares+=255;
   for (i=(-255); i <= 255; i++)
@@ -514,7 +530,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     Allocate image colormap.
   */
   if (AcquireImageColormap(image,number_clusters) == MagickFalse)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   i=0;
   for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
@@ -533,7 +549,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   exception=(&image->exception);
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+  #pragma omp parallel for schedule(static) shared(progress,status) \
     magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -647,10 +663,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,
-          2*image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1005,9 +1021,9 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -1827,7 +1843,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
           extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
           histogram[i]=(ssize_t *) RelinquishMagickMemory(histogram[i]);
         }
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename)
       }
   }
